@@ -11,13 +11,13 @@ class Node(object):
 
     def _execute(self, query, variables):
         if not self._context:
-            raise Exception('Expected context')
+            raise Exception("Expected context")
 
         return self._context.client._execute(query, variables)
 
     def __repr__(self):
-        return '{} (ID: {}) {}'.format(self.__class__.__name__, self.id, self._data)
+        return f"{self.__class__.__name__} (ID: {self.id}) {self._data}"
 
     @property
     def id(self):
-        return self._data['id']
+        return self._data["id"]
