@@ -1,13 +1,13 @@
 from obschart.api.node import Node
 
-imageFragment = '''
+imageFragment = """
 fragment Image on Image {
     id
     url
     width
     height
 }
-'''
+"""
 
 
 class Image(Node):
@@ -16,12 +16,12 @@ class Image(Node):
 
     @property
     def url(self):
-        return self._data['url']
+        return self._data["url"]
 
     @property
     def width(self):
-        return self._data['width']
+        return self._data["width"]
 
     @property
     def height(self):
-        return self._data['height']
+        return self._data["height"]
