@@ -2,7 +2,8 @@ from ..node import Node
 from .program_track_action import ProgramTrackAction, programTrackActionFragment
 from gql import gql
 
-programTrackActionResponseFragment = '''
+programTrackActionResponseFragment = (
+    """
 fragment ProgramTrackActionResponse on ProgramTrackActionResponse {
     id
     createdAt
@@ -11,7 +12,9 @@ fragment ProgramTrackActionResponse on ProgramTrackActionResponse {
     }
     response
 }
-''' + programTrackActionFragment
+"""
+    + programTrackActionFragment
+)
 
 
 class ProgramTrackActionResponse(Node):
@@ -20,27 +23,20 @@ class ProgramTrackActionResponse(Node):
 
     @property
     def action(self):
-        return ProgramTrackAction(self._data['action'], self._context)
+        return ProgramTrackAction(self._data["action"], self._context)
 
     @property
     def response(self):
-        return self._data['response']
+        return self._data["response"]
 
     def set_feedback(self, title: str, blocks):
         data = {
-            'type': 'multiStep',
-            'steps': [{
-                'name': title,
-                'action': {
-                    'type': 'blocks',
-                    'blocks': blocks
-                }
-            }]
+            "type": "multiStep",
+            "steps": [{"name": title, "action": {"type": "blocks", "blocks": blocks}}],
         }
 
         pma = self._context.client.create_program_track_action(data)
 
-        self._context.client.update_program_track_action_response(
-            self.id, pma.id)
+        self._context.client.update_program_track_action_response(self.id, pma.id)
 
         return pma
