@@ -1,21 +1,36 @@
 from gql import gql, Client
-from gql.transport.requests import RequestsHTTPTransport
+from .gql_client_transport import GqlClientTransport
 import json
-from obschart.api.nodes import Session, ProgramInvitation, ProgramTrackActionResponse, programTrackActionResponseFragment, ProgramTrackAction, programTrackActionFragment
+from obschart.api.nodes import (
+    Session,
+    Image,
+    imageFragment,
+    ProgramInvitation,
+    ProgramTrackActionResponse,
+    programTrackActionResponseFragment,
+    ProgramTrackAction,
+    programTrackActionFragment,
+)
 from obschart.api.context import Context
 from typing import Optional, Dict, Any, Callable
 import polling
 import datetime
 from .blocks_action_builder import BlocksActionBuilder
+import io
+from typing import Union
 
 OnResponseCallback = Callable[[ProgramTrackActionResponse], None]
 
 
 class ObschartClient:
-    def __init__(self, authentication_token: Optional[str] = None, api_url: Optional[str] = 'https://api.obschart.com/'):
+    def __init__(
+        self,
+        authentication_token: Optional[str] = None,
+        api_url: Optional[str] = "https://api.obschart.com/",
+    ):
         super().__init__()
 
-        transport = RequestsHTTPTransport(api_url, use_json=True)
+        transport = GqlClientTransport(api_url)
         self.gql_client = Client(transport=transport)
         self._context = Context(self)
 
@@ -24,26 +39,18 @@ class ObschartClient:
 
     def set_authentication_token(self, authentication_token: str):
         self.gql_client.transport.headers = self.gql_client.transport.headers or {}
-        self.gql_client.transport.headers['Authorization'] = 'Bearer {}'.format(
-            authentication_token)
+        self.gql_client.transport.headers["Authorization"] = f"Bearer {authentication_token}"
 
     def _execute(self, query, variables: Optional[Dict[str, Any]] = None):
-        variable_values = None
-
-        if variables:
-            variable_values = json.dumps(variables)
-
-        return self.gql_client.execute(
-            query, variable_values)
+        return self.gql_client.execute(query, variables)
 
     def _execute_mutation(self, query, input: Optional[Dict[str, Any]] = None):
-        variables = {
-            'input': input
-        }
+        variables = {"input": input}
         return self._execute(query, variables)
 
     def login(self, email: str, password: str):
-        query = gql('''
+        query = gql(
+            """
           mutation CreateSessionMutation($input: CreateSessionInput) {
             createSession(input: $input) {
               token
@@ -52,24 +59,22 @@ class ObschartClient:
               }
             }
           }
-          ''')
+          """
+        )
 
-        input = {
-            'password': password,
-            'email': email
-        }
-        result = self._execute_mutation(
-            query, input)
+        input = {"password": password, "email": email}
+        result = self._execute_mutation(query, input)
 
-        authentication_token = result['createSession']['token']
+        authentication_token = result["createSession"]["token"]
         self.set_authentication_token(authentication_token)
 
-        return Session(result['createSession']['session'], Context(self))
+        return Session(result["createSession"]["session"], Context(self))
 
     def poll_program_track_action_responses(self, filter):
         time = datetime.datetime.utcnow()
 
-        query = gql('''
+        query = gql(
+            """
           query OnResponseQuery($filter: ProgramTrackActionResponseFilterInput) {
             programTrackActionResponses(filter: $filter, first: 9999) {
               edges {
@@ -79,44 +84,33 @@ class ObschartClient:
               }
             }
           }
-        ''' + programTrackActionResponseFragment)
+        """
+            + programTrackActionResponseFragment
+        )
 
         filter = filter.copy()
-        filter.update({
-            'createdAt': {
-                'after': time.replace(
-                    tzinfo=datetime.timezone.utc).isoformat()
-            }
-        })
-        variables = {
-            'filter': filter
-        }
+        filter.update(
+            {"createdAt": {"after": time.replace(tzinfo=datetime.timezone.utc).isoformat()}}
+        )
+        variables = {"filter": filter}
 
         def poll():
             # print('Polling...')
             return self._execute(query, variables)
 
         def check_success(response):
-            return len(response['programTrackActionResponses']['edges']) > 0
+            return len(response["programTrackActionResponses"]["edges"]) > 0
 
-        response = polling.poll(
-            poll,
-            check_success=check_success,
-            step=1,
-            poll_forever=True
-        )
+        response = polling.poll(poll, check_success=check_success, step=1, poll_forever=True)
 
         responses: list[ProgramTrackActionResponse] = []
-        for edge in response['programTrackActionResponses']['edges']:
-            responses.append(ProgramTrackActionResponse(
-                edge['node'], self._context))
+        for edge in response["programTrackActionResponses"]["edges"]:
+            responses.append(ProgramTrackActionResponse(edge["node"], self._context))
 
         return responses
 
     def on_response(self, on_response_callback: OnResponseCallback):
-        responses = self.poll_program_track_action_responses({
-            'applicationId': {'eq': 'me'}
-        })
+        responses = self.poll_program_track_action_responses({"applicationId": {"eq": "me"}})
 
         for response in responses:
             on_response_callback(response)
@@ -124,7 +118,8 @@ class ObschartClient:
         return self.on_response(on_response_callback)
 
     def create_program_track_action(self, data: Any):
-        query = gql('''
+        query = gql(
+            """
           mutation CreateProgramTrackActionMutation($input: CreateProgramTrackActionInput!) {
             createProgramTrackAction(input: $input) {
               programTrackAction  {
@@ -132,18 +127,42 @@ class ObschartClient:
               }
             }
           }
-          ''' + programTrackActionFragment)
+          """
+            + programTrackActionFragment
+        )
 
-        input = {
-            'data': data
-        }
-        result = self._execute_mutation(
-            query, input)
+        input = {"data": data}
+        result = self._execute_mutation(query, input)
 
-        return ProgramTrackAction(result['createProgramTrackAction']['programTrackAction'], self._context)
+        return ProgramTrackAction(
+            result["createProgramTrackAction"]["programTrackAction"], self._context
+        )
+
+    def create_image(self, image: Union[str, Any]):
+        query = gql(
+            """
+          mutation CreateImageMutation($input: CreateImageInput!) {
+            createImage(input: $input) {
+              image  {
+                ...Image
+              }
+            }
+          }
+          """
+            + imageFragment
+        )
+
+        if isinstance(image, str):
+            image = open(image, "rb")
+
+        input = {"image": image}
+        result = self._execute_mutation(query, input)
+
+        return Image(result["createImage"]["image"], self._context)
 
     def create_program_invitation(self, program_id: str):
-        query = gql('''
+        query = gql(
+            """
           mutation CreateProgramInvitationMutation($input: CreateProgramInvitationInput!) {
             createProgramInvitation(input: $input) {
               programInvitation  {
@@ -151,18 +170,19 @@ class ObschartClient:
               }
             }
           }
-          ''')
+          """
+        )
 
-        input = {
-            'programId': program_id
-        }
-        result = self._execute_mutation(
-            query, input)
+        input = {"programId": program_id}
+        result = self._execute_mutation(query, input)
 
-        return ProgramInvitation(result['createProgramInvitation']['programInvitation'], self._context)
+        return ProgramInvitation(
+            result["createProgramInvitation"]["programInvitation"], self._context
+        )
 
     def send_program_invitation_sms(self, program_invitation_id: str, phone_number: str):
-        query = gql('''
+        query = gql(
+            """
           mutation SendProgramInvitationSmsMutation($input: SendProgramInvitationSmsInput) {
             sendProgramInvitationSms(input: $input) {
               programInvitation  {
@@ -170,19 +190,19 @@ class ObschartClient:
               }
             }
           }
-          ''')
+          """
+        )
 
-        input = {
-            'id':  program_invitation_id,
-            'phoneNumber':  phone_number
-        }
-        result = self._execute(
-            query, input)
+        input = {"id": program_invitation_id, "phoneNumber": phone_number}
+        result = self._execute(query, input)
 
-        return ProgramInvitation(result['sendProgramInvitationSms']['programInvitation'], self._context)
+        return ProgramInvitation(
+            result["sendProgramInvitationSms"]["programInvitation"], self._context
+        )
 
     def get_current_session(self):
-        query = gql('''
+        query = gql(
+            """
           query CurrentSessionQuery {
             currentSession {
               id
@@ -193,30 +213,33 @@ class ObschartClient:
               }
             }
           }
-          ''')
+          """
+        )
 
         result = self._execute(query)
 
-        return Session(result['currentSession'], self._context)
+        return Session(result["currentSession"], self._context)
 
     def get_program_track_action_response(self, id):
-        query = gql('''
+        query = gql(
+            """
           query ProgramTrackActionResponseQuery($id: ID) {
             programTrackActionResponse(id: $id) {
               ...ProgramTrackActionResponse
             }
           }
-          ''' + programTrackActionResponseFragment)
+          """
+            + programTrackActionResponseFragment
+        )
 
-        variables = {
-            'id': id
-        }
+        variables = {"id": id}
         result = self._execute(query, variables)
 
-        return ProgramTrackActionResponse(result['programTrackActionResponse'], self._context)
+        return ProgramTrackActionResponse(result["programTrackActionResponse"], self._context)
 
     def update_program_track_action_response(self, id: str, feedback_program_track_action_id: str):
-        query = gql('''
+        query = gql(
+            """
             mutation UpdateProgramTrackActionResponseMutation($input: UpdateProgramTrackActionResponseInput!)  {
                 updateProgramTrackActionResponse(input: $input) {
                     programTrackActionResponse {
@@ -224,15 +247,19 @@ class ObschartClient:
                     }
                 }
             }
-        ''' + programTrackActionResponseFragment)
+        """
+            + programTrackActionResponseFragment
+        )
 
         input = {
-            'id': id,
-            'feedbackProgramTrackActionId': feedback_program_track_action_id
+            "id": id,
+            "feedbackProgramTrackActionId": feedback_program_track_action_id,
         }
         result = self._context.client._execute_mutation(query, input)
 
-        return ProgramTrackActionResponse(result['updateProgramTrackActionResponse']['programTrackActionResponse'], self._execute)
+        return ProgramTrackActionResponse(
+            result["updateProgramTrackActionResponse"]["programTrackActionResponse"], self._execute,
+        )
 
     def build_feedback_data(self):
         return BlocksActionBuilder()
